# Scotty TNG - ping/snmp based realtime network monitoring
#
# $Id$
#
# Authors:
#   Thomas Liske <liske@ibh.de>
#   Andre Ellguth <ellguth@ibh.de>
#
# Copyright Holder:
#   2008 (C) IBH IT-Service GmbH [http://www.ibh.de/apt-dater/]
#
# License:
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this package; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
#

require 'snmp'

# This class monitors a single node using different +Sensor+ implementations.
class Node
  include DRbUndumped

  @@nodes = Hash.new

  attr_reader :name, :address, :sensors, :ringlists, :snmp

  def initialize(xmlnode)
    @name = xmlnode.attributes["name"]
    @address = xmlnode.attributes["address"]
    @ringlists = Hash.new

    c = REXML::XPath.first(xmlnode, "community/@value")
    c = REXML::XPath.first(xmlnode.root, "/config/defaults/community/@value") unless c

    case xmlnode.attributes["snmp-version"]
    when "1"
	version = :SNMPv1
    else
	version = :SNMPv2c
    end

    if c then
	@snmp = SNMP::Manager.new(:Host => @address,
				  :Community => c.to_s,
				  :Version => version,
				  :Timeout => 2,
				  :Retries => 2)
    end

    puts " " + name + " [" + address + "]"
    print "   "
    @sensors = Array.new
    xmlnode.elements.each("*") { |xmlsensor|
	next if xmlsensor.name == "community"
	sensor = Sensor.create(self, xmlsensor)
	if(sensor != nil)
	    print "#{sensor.name} "
	    @sensors.push(sensor)
	end
    }
    puts
    
    @@nodes[name] = self
  end

  def self.addNode(xmlnode)
    return Node.new(xmlnode)
  end

  def self.getNodes()
    return @@nodes
  end
end
